@@ -1,8 +1,8 @@
 //! Allows to listen to runtime events.
 
 use crate::Context;
-use evm_runtime::{CreateScheme, Transfer};
-use primitive_types::{H160, U256};
+use evm_runtime::{CreateScheme, ExitReason, Transfer};
+use primitive_types::{H160, H256, U256};
 
 environmental::environmental!(listener: dyn EventListener + 'static);
 
@@ -33,12 +33,37 @@ pub enum Event<'a> {
 		target: H160,
 		balance: U256,
 	},
+	Exit {
+		reason: &'a ExitReason,
+		return_value: &'a [u8],
+	},
+	TransactCall {
+		caller: H160,
+		address: H160,
+		value: U256,
+		data: &'a [u8],
+		gas_limit: u64,
+	},
+	TransactCreate {
+		caller: H160,
+		value: U256,
+		init_code: &'a [u8],
+		gas_limit: u64,
+		address: H160,
+	},
+	TransactCreate2 {
+		caller: H160,
+		value: U256,
+		init_code: &'a [u8],
+		salt: H256,
+		gas_limit: u64,
+		address: H160,
+	},
 }
 
-impl<'a> Event<'a> {
-	pub(crate) fn emit(self) {
-		listener::with(|listener| listener.event(self));
-	}
+// Expose `listener::with` to the crate only.
+pub(crate) fn with<F: FnOnce(&mut (dyn EventListener + 'static))>(f: F) {
+	listener::with(f);
 }
 
 /// Run closure with provided listener.
